/***************************************************************************
                          shmap.h  -  description
                             -------------------
    begin                : Wed Feb 6 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/



#ifndef _SHMAP_I
#define _SHMAP_I

#include <stdlib.h>

#include "shmap_value.h"

/*
        function module definition
*/
typedef struct shmap_module shmap_module;
typedef struct shmap_opts shmap_opts;
typedef struct shmap shmap;

/*
        well, currently we define all the module types
        over here
*/
#define SHMAP_TYPE_LL 1
#define SHMAP_TYPE_HM 2

// define the default type
#define SHMAP_TYPE_DEFAULT SHMAP_TYPE_LL

struct shmap_module {
        const char *   module_name;
        const char *   module_version;
        int            module_type;
        void *        (*shmap_init)(void *, void *);
        _shmap_value *(*shmap_get_value)(shmap *, const char *, int);
        int           (*shmap_add_value)(shmap *, const char *, void *, int); // map, key, value pointer, value options
        int           (*shmap_del_value)(shmap *, const char *);
        _shmap_value *(*shmap_get_sub)(shmap *, const char *, int, int, int); // map, key, keylen, pos, cached
        int           (*shmap_del_sub)(shmap*, const char *, int);
        int           (*shmap_clear)(shmap *);
        int           (*shmap_dispose)(shmap *);
        void *        (*shmap_cache_init)(void *);
        void          (*shmap_cache_clean)(void *);
        int           (*shmap_cache_dispose)(void *, void **);
        void          (*shmap_dump)(shmap *, int);
        };

struct shmap_opts {
        void * p_mem;      // the memory segment
        size_t size;       // the maximum size of map
        char   type;       // type of hashmap
        int    flags;      // flags for that hashmap
        char * p_mem_name; // the shared memory segment name (for client/server programs)
        };

struct shmap {
        void *toplevel;

        void * mem_segm;
        void * locks;

        unsigned int items;

        shmap_module * func;

        // some algorithms use data seeking cache for quicker access
        void * cache;
};

typedef enum{
        SHMAP_LOCK_READ,
        SHMAP_LOCK_WRITE
        } shmap_lock_mode;

typedef struct _shmap_lock shmap_lock;

/****************************************
 *
 *        the API functions itself
 *
 ****************************************/
shmap      * shmap_init ( shmap_opts * p_opts );
void       * shmap_get_value( shmap *, const char *, int );
int          shmap_get_int ( shmap *, const char *, int *, int);
int          shmap_add_value ( shmap *, const char *, const char *);
int          shmap_del_value ( shmap *, const char *);
char       * shmap_get_sub ( shmap *, const char *, int, int, int);
int          shmap_del_sub ( shmap *, const char *, int);
int          shmap_clear ( shmap * );
int          shmap_dispose ( shmap * );
size_t       shmap_items ( shmap * );
void       * shmap_memsegm ( shmap * );
void         shmap_dump ( shmap *, int );
size_t       shmap_available ( shmap * );
shmap_lock * shmap_lock_init ( void * p_mem, const char * path );
shmap_lock * shmap_lock_get ( shmap * map );
int          shmap_lock_set ( shmap * map, shmap_lock * lock );
int          shmap_lock_del ( shmap * map );
int          shmap_lock_dispose ( void * p_mem, shmap_lock * lock );
int          shmap_dolock ( shmap_lock * lock, shmap_lock_mode mode );
int          shmap_unlock ( shmap_lock * lock );

extern shmap_opts __def_opts;
#define SHMAP_DEFAULT &__def_opts

/****************************************
        aliases to API functions
 ****************************************/
#define shmap_add(m,k,v) shmap_add_value(m,k,v)
#define shmap_get(m,k) (char*)shmap_get_value(m,k,1)
#define shmap_del(m,k) shmap_del_value(m,k)

/*
        some utilities for general usage
*/
#define CHECK_NULL(m) if ( m == NULL ) return NULL;
#define CHECK_NULL0(m) if ( m == NULL ) return 0;
#define CHECK_NULL_(m) if ( m == NULL ) return;

#define DEBUG(t)        printf(t); fflush(stdout);

#include "shmap_ll.h"
#include "shmap_hm.h"

#endif
